//  
// cdcanl.C
//
// 

TFile *foutp;
TH1F  *hTrunc;


int cdcanl(Int_t run = 0)
{
  gROOT->Reset();
  
  if( strncmp(gSystem->HostName(),"ccjlc",5)  != 0 ) {
    if( strncmp(gSystem->Getenv("OSTYPE"),"hpux",4) == 0 ) {
      gSystem->Load("libCDCUtils.sl");
      gSystem->Load("libCDCRawData.sl");
      gSystem->Load("libCDCClusterMaker.sl");
      gSystem->Load("libCDCHitMaker.sl");
      gSystem->Load("libCDCTrackMaker.sl");
      gSystem->Load("../libCDCAnalysis.sl");
    } else {
      gSystem->Load("libCDCUtils.so");
      gSystem->Load("libCDCRawData.so");
      gSystem->Load("libCDCClusterMaker.so");
      gSystem->Load("libCDCHitMaker.so");
      gSystem->Load("libCDCTrackMaker.so");
      gSystem->Load("../libCDCAnalysis.so");
    }
  }
    
  JSFSteer *jsf = new JSFSteer();

  if (!run) run = jsf->Env()->GetValue("JSFSteer.RunNo",1034);  
  Char_t foname[64];
  sprintf(foname,"jsf%4.4dfixedwidth_varped.root",run);
  foutp = new TFile(foname,"RECREATE"); //  Output file.
  TFile & fout = *foutp;
  
  Char_t fname[64];
  // sprintf(fname,"run%4.4d.root",run);
  // sprintf(fname,"/proj/cdc/data/baby/2000/beam/unpacked/run%4.4d.root",run);
  sprintf(fname,"/proj/cdc/data/baby/2000/dedx/unpacked/run%4.4d.root",run);
  TFile file(fname); // Input file

  jsf->SetInput(file);
  jsf->SetOutput(fout);
  
  TDirectory *last = gDirectory;
  foutp->cd("/");
  hTrunc = new TH1F("Trunc","truncated mean",100,0.,8000.);
  last->cd();

  Int_t nevent=jsf->Env()->GetValue("JSFSteer.Nevent",100000);  
  Int_t minevt=1;
  Int_t maxevt=minevt+nevent;

  cout << " Run = " << run << " Nevents = " << nevent << endl;

  CDCFADCCalibration fadccalib("../../../data/fadccalib.dat");
  CDCClusterMaker *clust = new CDCClusterMaker(&fadccalib);
#if 0
  clust->UseConstPed();
#else
  clust->UseVarPed();
#endif
#if 1
  CDCCluster::SetSoftThresh(16.);
  CDCCluster::SetSplitThresh(20.);
#else
  CDCCluster::SetSoftThresh(20.);
  CDCCluster::SetSplitThresh(40.);
#endif
  CDCCluster::SetMaxNslots(100);

  CDCTimeToXRelation xtcalib("../../../data/xtcalib.dat");
  CDCXTDatum::SetSigmaMS(0.000);

  CDCHitMaker   *hit   = new CDCHitMaker(&xtcalib);
  CDCTrackMaker *track = new CDCTrackMaker();
#if 0
  CDCRawHist      *hist  = new CDCRawHist();
#endif
#if 0
  CDCClusterHist  *hist  = new CDCClusterHist();
#endif
#if 1
  CDCAnalysis     *hist  = new CDCAnalysis();

  //  hist->SetPhCut(5500.);        // Ph cut for Truncated mean

  hist->SetMode(0);		// (0,1) = (excl,incl) hit in question from fit
  hist->SetNwires(5);		// require Nwires or more in the analysis
  hist->SetChi2Cut(15.);	// require chi2 < xChi2Cut for reference track
  hist->SetNsigCut(5.);  	// window for residals
  hist->SetTrackRange(0,0);	// (i1,i2) = (1st,last) tracks to look at
  				// (0,0) to look at track closest to wire
  				// (1,1) to study 2-track separation 
#endif

  jsf->Initialize();
 
  // EventBuf set only after Initialize()!

  CDCTrackBuf *tkb = (CDCTrackBuf *)track->EventBuf();
#if 1
  tkb->SetNsigCut(5.);  	// window for residals
  tkb->SetChi2Cut(15.); 	// chi2 cut for tracks
  tkb->SetKCNsigCut(3.); 	// window for hit point poisoing
  tkb->SetDXpivotCut(0.05); 	// DX to be qualified as a pivot
#else
  tkb->SetNsigCut(10.); 	// window for residals
  tkb->SetChi2Cut(45.); 	// chi2 cut for tracks
  tkb->SetKCNsigCut(2.); 	// window for hit point poisoing
  tkb->SetDXpivotCut(0.05); 	// DX to be qualified as a pivot
#endif

  CDCHitBuf *htb = (CDCHitBuf *)hit->EventBuf();
#if 1
  htb->SetWidthCut(12.);
  htb->SetPeakCut(30.);
  htb->SetSumCut(200.);
#else
  htb->SetWidthCut(60.);
  htb->SetPeakCut(30.);
  htb->SetSumCut(1600.);
#endif

  CDCRawData *raw=(CDCRawData*)jsf->FindModule("CDCRawData");

  jsf->BeginRun(run);	// Set run number

  Int_t    nwires = 80;
  Double_t frac   = 0.7;
  Int_t    ntrunc = nwires*frac;
  Int_t    nhits  = 0;
  TObjArray sums(nwires);
  
  Double_t gain[]={1.11,1.08,1.02,1.23,1.0,
		     1.0,1.0,1.0,1.0,1.0};

  for(Int_t ev=minevt;ev<=maxevt;ev++){
    Int_t ib;
    if( ! (ib=jsf->GetEvent(ev)) ) break ;
#if 1
    ((CDCRawDataBuf*)raw->EventBuf())->GetHeader()->DebugPrint();
#endif
    if( ! jsf->Process(ev) ) break ;
#if 0
    ((CDCClusterBuf*)clust->EventBuf())->DebugPrint();
    ((CDCHitBuf*)hit->EventBuf())->DebugPrint();
    ((CDCTrackBuf*)track->EventBuf())->DebugPrint("Detailed");
#endif
    //for (Int_t i = 0; i < 5; i++) {
    for (Int_t i = 0; i < 5; i++) {
       Double_t phsum = hist->GetPhSum(i)/gain[i];
       cerr << "ev = " << ev << " wire = " << i << " phsum = " << phsum << endl;
       if (phsum > 0.) {
         nhits++;
	 sums.Add(new TObjNum(phsum));
	 if (nhits%nwires == 0) {
	   sums.Sort();
	   Double_t av = 0;
	   for (Int_t i = 0; i < ntrunc; i++) {
	     Double_t charge = ((TObjNum *)sums[i])->GetNum();
	     av += charge;
	     cerr << "i = " << i << " charge = " << charge << endl;
	   }
	   av /= ntrunc;
	   cerr << "truncated mean = " << av << endl;
           hTrunc->Fill(av);
	   sums.Delete();
	 }
       }
    }

    jsf->FillTree();
    jsf->Clear();
  }

  

  jsf->Terminate();
  fout.Write();

}
